
#include <sigc++/slot.h>
#include <iostream>
#include "MgeSystemTray.h"
#include "MainWindow.hh"
#include "gettext.h"

bool MgeSystemTray::onTrayClicked(GdkEventButton* event)
{
  bool retCode = false;

  cout << "onTrayClicked (button: " << event->button << ")" << endl;

  // Clear state
  // ?????
  // 	if (event->button == 1 && status_get_state () == STATUS_HIT) {
  //	 		status_set_state (STATUS_RUNNING);
  // 	}	

  // Clear state and toggle visibility
  if (event->button == 1 || event->button == 2)
	{
	  mainWinPtr->toggleVisibility();
	  retCode = true;
	}
  else 
	// Pop up tray context menu	
	if (event->button == 3)
	{
	  return onTrayMenu (event); //evtBox, event, data);
	}

	return retCode;
}

MgeSystemTray::MgeSystemTray(const Glib::RefPtr<Gdk::Pixbuf>& pixbuf)
: Egg::TrayIcon("PSP Tray Icon"), evtBox(0), imgStatus(0)
{
  toolTips.enable();

  evtBox = new Gtk::EventBox();

  if (pixbuf)
	imgStatus = new Gtk::Image(pixbuf);
  else
	{
	  std::cout << "=======================> No pixbuf given..." << std::endl;
	  imgStatus = new Gtk::Image(mainWinPtr->find_pixmap_file ("mgeops-psp-16.png"));
	}

  evtBox->add(*manage(imgStatus));

  // FIXME: Deal with background style problem
  //evtBox->set_visible_window (false);

  add(*manage(evtBox));

  setVisibility(false);

  // Connect menu callbacks
  //evtBox->button_release_event().connect(SigC::slot(*this, &MgeSystemTray::onTrayClicked));
  //evtBox->button_release_event().connect(SigC::slot(*this, &MgeSystemTray::onTrayClicked));

  //  evtBox->signal_button_release_event().connect(SigC::slot(*this, &MgeSystemTray::onTrayClicked));
  evtBox->signal_button_release_event().connect(sigc::mem_fun(*this, &MgeSystemTray::onTrayClicked));

  //gtklist->button_release_event().connect(slot(*this, &ListWindow::sigh_button_event));
//  g_signal_connect (G_OBJECT (eventbox), "button-release-event", G_CALLBACK (tray_clicked), NULL );
}

MgeSystemTray::~MgeSystemTray()
{
  ;
}

void MgeSystemTray::setPixbuf(const Glib::RefPtr<Gdk::Pixbuf>& pixbuf)
{
  imgStatus->set(pixbuf);
}

void MgeSystemTray::setImage(const Glib::RefPtr<Gtk::Image>& image)
{
  imgStatus->set(image->get_pixbuf());
}

void MgeSystemTray::setImage(const std::string imageName)
{
  imgStatus->set(Gtk::Image(imageName).get_pixbuf());
}

void MgeSystemTray::setTooltip(const Glib::ustring& tip, const Glib::ustring& private_tip)
{
  toolTips.set_tip(*evtBox, tip, private_tip);
}


void MgeSystemTray::setVisibility(bool visibility)
{
  bVisible = visibility;

  if (bVisible == true)
	  evtBox->show_all();
  else
	  evtBox->hide_all();

}

bool MgeSystemTray::onTrayMenu(GdkEventButton* event)
{
  Gtk::Menu* trayMenu;
  Gtk::ImageMenuItem* imgTrayMenuItem;
  Gtk::MenuItem* txtTrayMenuItem;
  Gtk::Image *imgTray;

  trayMenu = new Gtk::Menu();

  // Create the UPS status entry
  imgTray = new Gtk::Image(mainWinPtr->find_pixmap_file ("menu_status16.png"));
  trayMenu->items().push_back(Gtk::Menu_Helpers::ImageMenuElem(_("UPS status"), *imgTray));
  imgTrayMenuItem = (Gtk::ImageMenuItem *)&trayMenu->items().back();
  // Connect callbacks signals
  imgTrayMenuItem->signal_activate().connect(sigc::mem_fun(*this, &MgeSystemTray::onUpsMenuActivate));

  // Create a separator
  trayMenu->items().push_back(Gtk::Menu_Helpers::SeparatorElem());
  imgTrayMenuItem = (Gtk::ImageMenuItem *)&trayMenu->items().back();

  // Create the Settings entry
  imgTray = new Gtk::Image(mainWinPtr->find_pixmap_file ("menu_settings16.png"));
  trayMenu->items().push_back(Gtk::Menu_Helpers::ImageMenuElem(_("Settings"), *imgTray));
  imgTrayMenuItem = (Gtk::ImageMenuItem *)&trayMenu->items().back();
  // Connect callbacks signals
  imgTrayMenuItem->signal_activate().connect(sigc::mem_fun(*this, &MgeSystemTray::onSettingsMenuActivate));

  // Create the About entry
  imgTray = new Gtk::Image(mainWinPtr->find_pixmap_file ("mgeops-psp-16.png"));
  trayMenu->items().push_back(Gtk::Menu_Helpers::ImageMenuElem(_("About"), *imgTray));
  imgTrayMenuItem = (Gtk::ImageMenuItem *)&trayMenu->items().back();
  // Connect callbacks signals
  imgTrayMenuItem->signal_activate().connect(sigc::mem_fun(*this, &MgeSystemTray::onAboutMenuActivate));

  // Create the Help entry
  imgTray = new Gtk::Image(mainWinPtr->find_pixmap_file ("menu_help-16.png"));
  trayMenu->items().push_back(Gtk::Menu_Helpers::ImageMenuElem(_("Help"), *imgTray));
  imgTrayMenuItem = (Gtk::ImageMenuItem *)&trayMenu->items().back();
  // Connect callbacks signals
  imgTrayMenuItem->signal_activate().connect(sigc::mem_fun(*this, &MgeSystemTray::onHelpMenuActivate));

  // FIXME: needed? (standard on linux ; doesn't exist on win!)
  // Create the Exit entry
  imgTray = new Gtk::Image(mainWinPtr->find_pixmap_file ("exit16.png"));
  trayMenu->items().push_back(Gtk::Menu_Helpers::ImageMenuElem(_("Exit"), *imgTray));
  imgTrayMenuItem = (Gtk::ImageMenuItem *)&trayMenu->items().back();
  // Connect callbacks signals
  imgTrayMenuItem->signal_activate().connect(sigc::mem_fun(*this, &MgeSystemTray::onExitMenuActivate));

  // Display all
  trayMenu->show_all_children();

  // And popup the menu
  trayMenu->popup(event->button, event->time);

  return true;
}

void MgeSystemTray::onUpsMenuActivate()
{
  // Ensure that the main window is visible 
  mainWinPtr->setVisibility (true);

  mainWinPtr->setView("status");
}

void MgeSystemTray::onSettingsMenuActivate()
{
  // Ensure that the main window is visible 
  mainWinPtr->setVisibility (true);

  mainWinPtr->setView("settings");
}

void MgeSystemTray::onAboutMenuActivate()
{
  // Ensure that the main window is visible 
  mainWinPtr->setVisibility (true);

  mainWinPtr->setView("about");
}

void MgeSystemTray::onExitMenuActivate()
{
  // Self set invisibility to allow app. exit
  setVisibility (false);

  mainWinPtr->gtk_main_quit();
}

void MgeSystemTray::onHelpMenuActivate()
{
  // Ensure that the main window is visible 
  mainWinPtr->setVisibility (true);

  mainWinPtr->on_btHelp_released();
}
