// A class to provide support functions. 
/**
 * This class provides several support functions to deals with 
 * - graphics management
 * - determining the exact system type
 * - ...
 *
 * @author Arnaud Quette <arnaud.quette@mgeups.com>
 * @version 2005/01/11 Creation
 *
 * Copyright (C) 2004 - 2005 by MGE UPS SYSTEMS
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2.
 *  Please, read the COPYING file provided with this software for the full text
 */

#ifndef _MGEUTILS_HH
#  include <glibmm.h>
#  include <gtkmm.h>
#  define _MGEUTILS_HH

#include <string>

class MgeUtils;
extern MgeUtils *mgeUtilsPtr;

class MgeUtils
{
 public:
  // 
  MgeUtils();
 

  std::string findPixmapFile (const std::string filename);

  enum SystemType
	{
	  SYSTEM_TYPE_UNKNOWN =	0,	/// undetermined
	  SYSTEM_TYPE_DEBIAN,				/// Debian and derived
	  SYSTEM_TYPE_MDK,					/// Mandrakelinux
		SYSTEM_TYPE_MDK_2007,			/// Mandrakelinux 2007
	  SYSTEM_TYPE_SUSE,					/// SUSE
	  SYSTEM_TYPE_RHFC					/// RedHat / Fedora Core
	};

  // Return the System type, according the above values
  int getSystemType();

  // Check for a previous PSP instance running
  bool checkPrevInstance();
  int  getOldPid();
  void destroyAppLock();

  bool signalQuit();
  
  // Delete a file
  void deleteFile(const Glib::ustring *fileName);

  enum userModes {
   USER_MODE = 0, 
   ADMIN_MODE
  };

  bool setAdminMode(int mode) { userMode = mode; };
  int getAdminMode() { return userMode; };
  bool switchToRoot(Glib::ustring *command);

	void openURL(const char *url);
  void launchHelpViewer();
  void displayInfoDialog(Glib::ustring text, Glib::ustring icon="warning48.png");
  void displayInfoDialog_new(Gtk::MessageType type, char* text, bool exitOnDlgClose=false);

 protected:
  void addPixmapDirectory (const std::string &newDirectory);

 private:
  std::list<std::string> pixmapsDirectories;
  bool createAppLock();
  Glib::ustring *lockFileName;
  Glib::RefPtr<Glib::IOChannel> lockFile;
  int userMode;
  
};

#endif
