
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <stdio.h>
#include <upsclient.h>

// for serial probing
#include <grp.h>
#include <pwd.h>
#include <ctype.h>
#include <sys/file.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/termios.h>
#include <sys/ioctl.h>
#include <errno.h>
extern int errno;

// for USB probing
#include <usb.h>

#include "config.h"
#include "NutClient.h"

// FIXME: not clean
#include "MainWindow.hh"

#define MAX_SLEEP_RETRY	10

/*
 * Standard gettext macros.
 */
#ifdef ENABLE_NLS
#  include <libintl.h>
#  undef _
#  define _(String) dgettext (GETTEXT_PACKAGE, String)
#  ifdef gettext_noop
#    define N_(String) gettext_noop (String)
#  else
#    define N_(String) (String)
#  endif
#else
#  define textdomain(String) (String)
#  define gettext(String) (String)
#  define dgettext(Domain,Message) (Message)
#  define dcgettext(Domain,Message,Type) (Message)
#  define bindtextdomain(Domain,Directory) (Domain)
#  define _(String) (String)
#  define N_(String) (String)
#endif

NutClient::NutClient ()
  : Tool("NUTClient")
{
	std::cout << "new NutClient" << std::endl;
	
	//  if (upscli_splitname(DFL_UPSNAME, &upsname, &hostname, &port) != 0)
	//	clean_exit(&ups, upsname, hostname, EXIT_FAILURE);
	upsName = "mgeups";
	
	// Initialize members
	upsModel = new Glib::ustring();
	upsDescription = new Glib::ustring();
	commStatus = new Glib::ustring();
	upsStatus = new Glib::ustring();
	battStatus = new Glib::ustring();
	outputLoad = -1;
	outputStatus = new Glib::ustring();
	outlet1Status = new Glib::ustring();
	outlet2Status = new Glib::ustring();
	batteryCharge = -1;  // in percent, without the " %" suffix
	batteryLowCharge = -1; //0;
	batteryRuntime = -1; // in seconds
	testResult = new Glib::ustring();
	m_hostName = new Glib::ustring("localhost"); // set the default host
	
	nutVersion = new Glib::ustring();
	driverName = new Glib::ustring();
	driverVersion = new Glib::ustring();
	
	sleepRetry = 0;
	lastConnexionTime = time(NULL);

	// Connect to NUT' server
	connect();

}

#define MGE_CONNECT_DELAY 500000

void setline (int fd, int set)
{
	int dtr = TIOCM_DTR;
	int rts = TIOCM_RTS;

	if (set == 1)
	  {
		ioctl (fd, TIOCMBIC, &dtr);
		ioctl (fd, TIOCMBIS, &rts);
	  }
	else 
	  {
		ioctl (fd, TIOCMBIS, &dtr);
		ioctl (fd, TIOCMBIC, &rts);
	  }
}

// return the type of UPS found, and its communication port
int NutClient::probeUPS(Glib::ustring* comPort)
{
  bool deviceFound = false;
  int retValue = UPS_TYPE_NONE;
  char replyData[10];
  int retry;
  int fd, ttyNumber=0;
  ostringstream oss;

  // FIXME: delegate probe to the drivers (flag "-p" probe)

// <FIXME: Need LIBUSB flag> ie HAVE_LIBUSB
  // Probe USB bus
  usb_dev_handle *udev;
  struct usb_bus *bus;
  struct usb_device *dev;

  usb_init();
  usb_find_busses();
  usb_find_devices();

  for (bus = usb_busses; bus && !deviceFound; bus = bus->next) {
    for (dev = bus->devices; dev && !deviceFound; dev = dev->next) {
      
      udev = usb_open(dev);
      if (udev) {

		// Check the VendorID for matching flag
        if (dev->descriptor.idVendor == 0x463) {
		  deviceFound = true;
		  retValue = UPS_TYPE_USB;
		  comPort->assign("auto");
		}

		usb_close(udev);
	  }
	}
  }
// </FIXME: Need LIBUSB flag>

  // PnP Probe serial bus (if nothing on USB)
  if (!deviceFound)
	{
	  struct	termios	tio;
	  // Serial port opening (FIXME: loop on /dev/ttyS{1,8} => configurable)
	  while ( (!deviceFound) && (ttyNumber < 4) )
		{
		  comPort->clear ();
		  // FIXME: to be adapted for other platforms
		  oss.str("");
		  oss << "/dev/ttyS" << ttyNumber;
		  comPort->assign(oss.str());
		  std::cout << "Probing UPS on port: " << comPort->data() << std::endl;

		  fd = open(comPort->data(), O_RDWR | O_NOCTTY | O_EXCL | O_NONBLOCK);
		  //fd = open("/dev/ttyS0", O_RDWR | O_NOCTTY | O_EXCL | O_NONBLOCK);
		  if (fd > 0)
			{
			  // General serial port init (for both SHUT and UTalk)
			  if (tcgetattr(fd, &tio) == 0)
				{
				  tio.c_cflag = CS8 | CLOCAL | CREAD;
				  tio.c_iflag = IGNPAR;
				  tio.c_oflag = 0;
				  tio.c_lflag = 0;
				  tio.c_cc[VMIN] = 1;
				  tio.c_cc[VTIME] = 0;
				  cfsetispeed(&tio, B2400);
				  cfsetospeed(&tio, B2400);
				  tcflush(fd, TCIFLUSH);
				  tcsetattr(fd, TCSANOW, &tio);
				  
				  //tcflush(fd, TCIFLUSH); // flush received, unread data

				  // Try a SHUT sequence first (for bi-protocol Unit)
				  setline (fd, 1); // FIXME: needed?
				  for (retry = 0; retry < 3; retry++)
					{
					  if (sendShut(fd, "") != -1)
						{
						  std::cout << "Found a SHUT device" << std::endl;
						  deviceFound = true;
						  retValue = UPS_TYPE_SHUT;
						}
					}
				  
				  // FIXME: is setline (0); sufficient to restore line??
				  if (!deviceFound)
					{
					  setline (fd, 0);
					  for (retry = 0; retry < 3; retry++)
						{
						  // Try an UTalk sequence (Z, Z, Ax 1, Si)
						  if (sendUtalk(fd, "Z") != -1)
							std::cout << "UTalk data sent" << std::endl;
						  else
							std::cout << "UTalk data sent failed" << std::endl;
						  
						  if (sendUtalk(fd, "Z") != -1)
							std::cout << "UTalk data sent" << std::endl;
						  else
							std::cout << "UTalk data sent failed" << std::endl;
						  if (sendUtalk(fd, "Ax 1") != -1)
							std::cout << "	UTalk data sent" << std::endl;
						  else
							std::cout << "UTalk data sent failed" << std::endl;
						  
						  sendUtalk(fd, "Bl ?");
						  sendUtalk(fd, "Sm ?");
						  sendUtalk(fd, "Sn ?");
						  
						  usleep(MGE_CONNECT_DELAY);

						  if (sendUtalk(fd, "Si", true, &replyData[0]) != -1)
							{
							  std::cout << "Received UTalk data" << std::endl;
							  deviceFound = true;
							  retValue = UPS_TYPE_UTALK;
							}
						  else
							std::cout << "Nothing received (" << errno << ")" << std::endl;
						} // fin for (retry
					}	// if (!deviceFound)
				}	// fin tcgetattr ...
			  close(fd);
			  ttyNumber++;
			} // fin if (fd
		  else
			std::cout << "Error opening the communication port" << std::endl;
			
		} // fin while
	}
  return retValue;
}

#define SHUT_SYNC               0x16

// return the code read, 1 otherwise (if OK), -1 else
int NutClient::sendShut(int fd, const char *data, bool reply)
{
  int retry;
  unsigned char shutBuffer[1];
  struct timeval serial_timeout;
  fd_set readfs;
  int readen = 0;
  int rc = 0;
  
  FD_ZERO (&readfs);
  FD_SET (fd, &readfs);
  
  serial_timeout.tv_usec = 0;
  serial_timeout.tv_sec = 2; // FIXME: was 1;

  shutBuffer[0] = SHUT_SYNC;
  for (retry = 0; retry < 4; retry++) {
	if (write (fd, shutBuffer, 1) == -1)
	  {
		std::cout << "Failed sending SHUT SYNC" << std::endl;
		continue;
	  }
	else
	  std::cout << "SHUT SYNC sent" << std::endl;
	// blank the buffer
	shutBuffer[0] = '\0';

	rc = select (fd + 1, &readfs, NULL, NULL, &serial_timeout);
	if (rc == 0)
	  {
		std::cout << "Timeout" << std::endl;
		continue;			/* timeout */
	  }
	else
	  {
		if (FD_ISSET (fd, &readfs))
		  {
			// and read the device's answer
			if (read (fd, &shutBuffer[0], 1) == -1) // FIXME: 1 sec de timeout
			  {
				std::cout << "Failed reading response: " << errno << std::endl;
				continue;
			  }
			else
			  {
				if (shutBuffer[0] == SHUT_SYNC) 
				  {
					std::cout << "SHUT SYNC ok" << std::endl;
					return 1;
				  }
				else
				  {
					printf ("received %x\n", shutBuffer[0]);
					std::cout << "SHUT SYNC !ok" << shutBuffer[0] << std::endl;
				  }
			  }
		  }
	  }
  }
  return -1;
}

#define MGE_COMMAND_ENDCHAR "\r\n"
#define MGE_CHAR_DELAY 500
#define MGE_REPLY_DELAY 1000

// return the code read, 1 otherwise (if OK), -1 else
int NutClient::sendUtalk(int fd, const char *data, bool reply, char *replyData)
{
  const char *p;
  char Resp[32]="";
  int ret;
  fd_set rfds;
  struct timeval	tv; 

  // flush received, unread data
  tcflush(fd, TCIFLUSH);

  // send command
  for (p = data; *p; p++) {
	if ( isprint(*p & 0xFF) )
	  printf("mge_command: sending [%c]\n", *p);
	else
	  printf("mge_command: sending [%02X]\n", *p);
	
	if (write(fd, p, 1) != 1)
			return -1;
	
	usleep(MGE_CHAR_DELAY);
  }

  // send terminating string
  for (p = MGE_COMMAND_ENDCHAR; *p; p++) {
	if ( isprint(*p & 0xFF) )
	  printf("mge_command: sending [%c]\n", *p);
	else
	  printf("mge_command: sending [%02X]\n", *p);

	if (write(fd, p, 1) != 1)
	  return -1;
	
	usleep(MGE_CHAR_DELAY);
  }

  if (reply == true)
	{
	  usleep(MGE_REPLY_DELAY);

	  FD_ZERO(&rfds);
	  FD_SET(fd, &rfds);
	  
	  tv.tv_sec = 3;
	  tv.tv_usec = 0;
	  
	  ret = select(fd + 1, &rfds, NULL, NULL, &tv);

	  if (ret == -1)
		return -1;

	  if (ret == 0)
		  return -1;	/* timeout */

	  //	  ret = read(fd, buf, buflen);
	  //bytes_rcvd = ser_get_line(upsfd, reply, replylen,
	  //MGE_REPLY_ENDCHAR, MGE_REPLY_IGNCHAR, 3, 0);
	  return read (fd, replyData, 4); // only read Si.Data1
	}

  return 1;
}

int NutClient::getDesc(std::string *upsDescription)
{
  int	ret = 0;
  unsigned int	numq, numa;
  const	char	*query[4];
  char	**answer;

	std::cout << "NutClient::getDesc" << std::endl;

  query[0] = "UPSDESC";
  query[1] = "mgeups"; //upsName;
  numq = 2;
		
  // ret = upscli_list_start(ups, numq, query);
  // int upscli_get(UPSCONN *ups, unsigned int numq, const char **query,
  //	              unsigned int *numa, char ***answer);
  ret = upscli_get(&nutConnexion, numq, query, &numa, &answer);


  if (ret < 0) {
		std::cout << "FAILURE" << std::endl;

		/* new var and old upsd?  try to explain the situation */
		if (upscli_upserror(&nutConnexion) == UPSCLI_ERR_UNKCOMMAND) {
		  std::cerr << "Error: variable unknown (old upsd detected)" << std::endl;
		  return EXIT_FAILURE;
		}
		
		std::cerr <<  "Error: " << upscli_strerror(&nutConnexion) << std::endl;
	 	return EXIT_FAILURE;
  }
  
  if (numa < numq) {
		std::cerr << "FAILURE: insufficient data" << std::endl;
// 	fprintf(stderr, "Error: insufficient data "
// 			"(got %d args, need at least %d)\n", numa, numq);
 		return EXIT_FAILURE;
  }
  
  std::cout << "Desc for mgeups: " << answer[2] << "\n" << std::endl;

  upsDescription->assign(answer[2]);

  return EXIT_SUCCESS;
}

int NutClient::getVar(char *varName, std::string *varValue)
{
  int	ret = 0;
  unsigned int	numq, numa;
  const	char	*query[4];
  char	**answer;

	std::cout << "NutClient::getVar" << std::endl;

  query[0] = "VAR";
  query[1] = "mgeups"; //upsName;
  query[2] = varName;
  numq = 3;

  // ret = upscli_list_start(ups, numq, query);
  // int upscli_get(UPSCONN *ups, unsigned int numq, const char **query,
  //	              unsigned int *numa, char ***answer);
  ret = upscli_get(&nutConnexion, numq, query, &numa, &answer);


  if (ret < 0) {
		std::cout << "Failure retrieving " << varName << std::endl;

		/* new var and old upsd?  try to explain the situation */
		if (upscli_upserror(&nutConnexion) == UPSCLI_ERR_UNKCOMMAND) {
		  std::cerr << "Error: variable unknown (old upsd detected)" << std::endl;
		  return EXIT_FAILURE;
		}
		
		std::cout << "Error: " << upscli_strerror(&nutConnexion) << "(" << upscli_upserror(&nutConnexion) << ")" << 		std::endl;
	 	return -upscli_upserror(&nutConnexion); //EXIT_FAILURE;
  }
  
  if (numa < numq) {
		std::cerr << "FAILURE" << std::endl;
// 	fprintf(stderr, "Error: insufficient data "
// 			"(got %d args, need at least %d)\n", numa, numq);
 		return EXIT_FAILURE;
  }
  
  std::cout << "Value for " << varName << ": " << answer[3] << std::endl;

  varValue->assign(answer[3]);

  return EXIT_SUCCESS;

}

void NutClient::refreshVars()
{
  std::string tmpString;
  
  for(;;)
	{	  
	  /// Check the connection, and reconnect if necessary
	  if (commStatus->compare("OK") != 0)
			connect();

	  std::cout << "Communication status = " << commStatus->data() << std::endl;

	  /// Acquire Write Lock on data
	  std::cout << "waiting for writer lock" << std::endl;
	  dataLock->writer_lock ();
	  std::cout << "got writer lock" << std::endl;

	  if (getVar("ups.model", &tmpString) == EXIT_SUCCESS)
		{
		  upsModel->assign(tmpString);
		  commStatus->assign("OK");
		}
	  else
		{ 
		  commStatus->assign("None");
		  // Release Write Lock on data
		  dataLock->writer_unlock();
		  std::cout << "writer lock released" << std::endl;
		  continue;
		}
	  
	  if (getDesc(&tmpString) == EXIT_SUCCESS)
		{
		  upsDescription->assign(tmpString);
		  commStatus->assign("OK");
		}
	  else
		{ 
		  commStatus->assign("None");
		  // Release Write Lock on data
		  dataLock->writer_unlock();
		  std::cout << "writer lock released" << std::endl;
		  continue;
		}
	  
	  /// Get NUT release information
	  // FIXME: only retrieve once (test if empty)
	  if (getVar("driver.version", &tmpString) == EXIT_SUCCESS)
		{
		  nutVersion->assign(tmpString);
		  commStatus->assign("OK");
		  if (getVar("driver.name", &tmpString) == EXIT_SUCCESS)
			{
			  driverName->assign(tmpString);
		  }
		  if (getVar("driver.version.internal", &tmpString) == EXIT_SUCCESS)
			{
			  driverVersion->assign(tmpString);
			}
		}
	  else 
		{ 
		  commStatus->assign("None");
		  // Release Write Lock on data
		  dataLock->writer_unlock();
		  std::cout << "writer lock released" << std::endl;
		  continue;
		}
	  
	  if (getVar("ups.status", &tmpString) == EXIT_SUCCESS)
		{
		  /// Process status data
		  std::cout << "ups.status = " << tmpString << std::endl;
		  Glib::ustring::size_type strPos = tmpString.find("OL");
		  upsStatus->assign((int)strPos != -1 ? "AC Power" : "Battery");
		  strPos = tmpString.find("DISCHRG");
		  battStatus->assign((int)strPos != -1 ? _("Discharging") : _("Charging"));
		  
		  /// Process Main Output status
		  strPos = tmpString.find("BYPASS");
		  if ((int)strPos != -1)
			outputStatus->assign("Bypass");
		  else
			{
			  strPos = tmpString.find("OFF");
			  // FIXME: also check? outlet.0.switch == on | off
			  outputStatus->assign((int)strPos != -1 ? "Off" : "On");
			  std::cout << "outputStatus = " << outputStatus->data() << std::endl;
			  commStatus->assign("OK");
			}

		  /// Process load on output	
		  if (getVar("ups.load", &tmpString) == EXIT_SUCCESS)
			{
			  outputLoad = atoi(tmpString.data());
			}

		  /// Process Outlets status	
		  if (getVar("outlet.1.switch", &tmpString) == EXIT_SUCCESS)
			{
			  outlet1Status->assign(tmpString.data());
			  std::cout << "outletStatus: 1 = " << outlet1Status->data() << std::endl;
			}
		  else // Check with the new NUT outlet
			{
				if (getVar("outlet.1.status", &tmpString) == EXIT_SUCCESS)
				{
				  outlet1Status->assign(tmpString.data());
				  std::cout << "outletStatus: 1 = " << outlet1Status->data() << std::endl;
				}
			  else // No outlet1 to manage
				{
				  outlet1Status->assign("none");
				}
			}

		  if (getVar("outlet.2.switch", &tmpString) == EXIT_SUCCESS)
			{
			  outlet2Status->assign(tmpString.data());
			  std::cout << "outletStatus: 2 = " << outlet2Status->data() << std::endl;
			}
		  else
		  {
				if (getVar("outlet.2.status", &tmpString) == EXIT_SUCCESS)
				{
				  outlet2Status->assign(tmpString.data());
				  std::cout << "outletStatus: 2 = " << outlet2Status->data() << std::endl;
				}
				else // No outlet2 to manage
				{
					outlet2Status->assign("none");
				}
			}

		  /// Process Battery charge level
		  if (getVar("battery.charge", &tmpString) == EXIT_SUCCESS)
			{
			  batteryCharge = atoi(tmpString.data());
			}
		  
		  /// Process Low Battery charge level
		  if (getVar("battery.charge.low", &tmpString) == EXIT_SUCCESS)
			{
			  batteryLowCharge = atoi(tmpString.data());
			}

		  /// Process Remaining backup time	
		  if (getVar("battery.runtime", &tmpString) == EXIT_SUCCESS)
			{
			  batteryRuntime = atoi(tmpString.data());
			}
		  
		  /// Process "Last periodic test"
			/// FIXME: how to translate this value?
		  if (getVar("ups.test.result", &tmpString) == EXIT_SUCCESS)
			{
//   { 1, "Done and passed" },
//   { 2, "Done and warning" },
//   { 3, "Done and error" },
//   { 4, "Aborted" },
//   { 5, "In progress" },
//   { 6, "No test initiated" },
			  testResult->assign(_(tmpString.data()));
			}
		  else
			{
			  testResult->assign(_("Unknown"));
			}
	  	  
		  /// Assign the communication status
		  commStatus->assign("OK");
		}
	  else 
		{ 
		  commStatus->assign(_("Fault"));
		  // Release Write Lock on data
		  dataLock->writer_unlock();
		  std::cout << "writer lock released" << std::endl;
		  continue;
		}
	  
	  /// Release Write Lock on data
	  dataLock->writer_unlock();
	  std::cout << "writer lock released" << std::endl;

	  /// Wait 5 seconds before the next loop
	  Glib::usleep(5000000);
	}
}

bool NutClient::connect()
{
	int retCode = 0;
  std::string tmpString;
	
	std::cout << "(Re)connecting NutClient" << std::endl;
	
	disconnect();
	
	// Incremental sleeping before retrying
	if (sleepRetry < MAX_SLEEP_RETRY)
	{
		std::cerr << "sleeping" << "\n" << std::endl;
		sleepRetry++;
	}
	else
		sleepRetry = 1;
	
	// Check if we can retry to connect
	if (time(NULL) > (lastConnexionTime + sleepRetry))
	{
		if ((retCode = upscli_connect(&nutConnexion, "localhost", 3493, UPSCLI_CONN_TRYSSL)) < 0)
		{
			std::cerr << "Connection error: " << upscli_strerror(&nutConnexion) << "\n" << std::endl;
			commStatus->assign("None");
		}
		else
		{
			// Test the availability of the UPS
			if (getDesc(&tmpString) != EXIT_SUCCESS) {
				std::cerr << "Connection error with the UPS (mgeups): " << upscli_strerror(&nutConnexion) << std::endl;
				commStatus->assign("None");
			}
			else {
				commStatus->assign("OK");
				std::cout << "Connected to localhost" << std::endl;
			}
		}
		lastConnexionTime = time(NULL);
	}
	else
	{
		std::cout << "Sleeping " << (lastConnexionTime + sleepRetry) - time(NULL);
		std::cout << " seconds more before trying to reconnect" << std::endl;
		sleep (1);
	}

	// Leave time to refresh the GUI
	while(Gtk::Main::events_pending())
		Gtk::Main::iteration();
	

	return (retCode >= 0) ;
}
