// A Network UPS Tools Client interface. 
/**
 * This class allows to access NUT data server and manipulate
 * NUT data.
 *
 * @author Arnaud Quette <arnaud.quette@mgeups.com>
 * @version 2004/12/21 Creation
 *
 * Copyright (C) 2004 - 2005 by MGE UPS SYSTEMS
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2.
 *  Please, read the COPYING file provided with this software for the full text
 */

#ifndef _NUTCLIENT_HH
#  include <glibmm.h>
#  define _NUTCLIENT_HH

#include <config.h>
 
// NUT related includes
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <stdio.h>
#include <upsclient.h>

#include "tool.h"

#define DFL_UPSNAME "mgeups@localhost"
//#define EXIT_FAILURE -1
//#define EXIT_OK 1

// FIXME: make an enum in NutClient
#define UPS_TYPE_NONE  0
#define UPS_TYPE_USB   1
#define UPS_TYPE_SHUT  2
#define UPS_TYPE_UTALK 3


class NutClient : public Tool
{
 public:
  NutClient ();
  ~NutClient () { };
  
	
  // Autodetection routine (return upsType found)
  int probeUPS(Glib::ustring* comPort);
  int sendUtalk(int fd, const char *data, bool reply=false, char *replyData=NULL);
  int sendShut(int fd, const char *data, bool reply=false);

  // Accesseurs
  int getVar(char *varName, std::string *varValue);
  char *getComStatus();
  void setComStatus();

  // To access safely all variables (consumer part)
  void lockVars() {
		std::cout << "R ==> waiting for reader lock" << std::endl;
		dataLock->reader_lock () ;
		std::cout << "R ==> got reader lock" << std::endl;
  };
  // To release all variables once read (consumer part)
  void releaseVars() {
		dataLock->reader_unlock () ;
		std::cout << "R ==> reader lock released" << std::endl;
  };

  void createLock(Glib::StaticRWLock *lockVariable) {
		dataLock = lockVariable;
  };

  void setHost(Glib::ustring *hostName)
	{ m_hostName = hostName; }

  void refreshVars(); // threadly refresh the below variables
  Glib::Thread *refreshDataThread;
  //  Glib::StaticRWLock *dataLock; // = GLIBMM_STATIC_RW_LOCK_INIT;

  inline bool connect();
  inline bool disconnect() {
		return (upscli_disconnect(&nutConnexion) == 0) ;
  };

	// private:
	char*								upsName; // default to mgeups[@localhost]
	Glib::ustring*			upsModel;
	Glib::ustring*			upsDescription;
	Glib::ustring*			commStatus;
	Glib::ustring*			upsStatus;
	Glib::ustring*			battStatus;
	int									outputLoad;
	Glib::ustring*			outputStatus;
	Glib::ustring*			outlet1Status;
	Glib::ustring*			outlet2Status;
	int									batteryCharge;		// in percent
	int									batteryLowCharge;	// in percent
	int									batteryRuntime;	// in seconds
	Glib::ustring*			testResult;
	
	Glib::ustring*			nutVersion;
	Glib::ustring*			driverName;
	Glib::ustring*			driverVersion;

private:
	int									getDesc(std::string *upsDescription);
	int									needApply;         /* 0 if no, 1 if yes */
	UPSCONN_t						nutConnexion;
	Glib::StaticRWLock	*dataLock; // = GLIBMM_STATIC_RW_LOCK_INIT;
	int									sleepRetry;
	time_t							lastConnexionTime;
	Glib::ustring*	m_hostName;
};

#endif
