
#include "config.h"
#include "NutConfig.h"

#define MNC_CFG_HEADER "# Generated by " MFR_NAME " - Personal Solution Pac ("PACKAGE", version "VERSION")\n\n"

NutConfig::NutConfig ()
{ 
  driverType = UPS_TYPE_NONE;
  communicationPort = new Glib::ustring("");
  shutdownTimer = -1;
  shutdownBatteryLevel = -1;
  shutdownBatteryRuntime = -1;
  systemType = pspUtils.getSystemType();
  displaySystray = true; // FIXME: false;
  shutdownType = -1; //SD_STANDARD;
  shutdownCustomCmd = new Glib::ustring(""); ///sbin/shutdown -h +0");

  switch(systemType)
	{
	case MgeUtils::SYSTEM_TYPE_DEBIAN:
	  nutCfgDir = new Glib::ustring("/etc/nut/");
	  nutUser = new Glib::ustring("nut");
	  break;
	case MgeUtils::SYSTEM_TYPE_RHFC:
	  nutCfgDir = new Glib::ustring("/etc/ups/");
	  nutUser = new Glib::ustring("nut");
	  break;
	case MgeUtils::SYSTEM_TYPE_SUSE:
	  nutCfgDir = new Glib::ustring("/etc/ups/");
	  nutUser = new Glib::ustring("daemon"); //nobody");
	  break;
	case MgeUtils::SYSTEM_TYPE_MDK:
	case MgeUtils::SYSTEM_TYPE_MDK_2007:
	default:
	  nutCfgDir = new Glib::ustring("/etc/ups/");
	  nutUser = new Glib::ustring("ups");
	  break;
	}
  bIsNutConfigured = false; //isNutConfigured();

  // FIXME: load serialised config from wizard.conf
  // ...
  readConfiguration();
}

NutConfig::NutConfig (int drvType, Glib::ustring* comPort)
{
  // FIXME: (h2) first recall default constructor?!

  driverType = drvType;
  communicationPort = new Glib::ustring(*comPort);
  shutdownTimer = -1;
  shutdownBatteryLevel = -1;
  shutdownBatteryRuntime = -1;
  bIsNutConfigured = true;
  systemType = pspUtils.getSystemType();
  displaySystray = true; // FIXME: false;
  shutdownType = -1; //SD_STANDARD;
  shutdownCustomCmd = new Glib::ustring(); ///sbin/shutdown -h +0");

  switch(systemType)
	{
	case MgeUtils::SYSTEM_TYPE_DEBIAN:
	  nutCfgDir = new Glib::ustring("/etc/nut/");
	  nutUser = new Glib::ustring("nut");
	  break;
	case MgeUtils::SYSTEM_TYPE_RHFC:
	  nutCfgDir = new Glib::ustring("/etc/ups/");
	  nutUser = new Glib::ustring("nut");
	  break;
	case MgeUtils::SYSTEM_TYPE_SUSE:
	  nutCfgDir = new Glib::ustring("/etc/ups/");
	  nutUser = new Glib::ustring("daemon"); //nobody");
	  break;
	case MgeUtils::SYSTEM_TYPE_MDK:
	case MgeUtils::SYSTEM_TYPE_MDK_2007:
	default:
	  nutCfgDir = new Glib::ustring("/etc/ups/");
	  nutUser = new Glib::ustring("ups");
	  break;
	}
  // FIXME: load serialised config from wizard.conf
  // ...
  readConfiguration();

}

bool NutConfig::isNutConfigured()
{ 
	bool retValue = false;
	Glib::ustring lineRead;
	int test;

  // FIXME: to be implemented (read /etc/nut/wizard.conf ?)
  Glib::RefPtr<Glib::IOChannel> wizardConfFile;

  /// Open the wizard.conf file
  try
	{
	  wizardConfFile = Glib::IOChannel::create_from_file(Glib::ustring(*nutCfgDir + Glib::ustring("wizard.conf")).raw(), "r");
	}
  catch (Glib::FileError& e)
	{
	  return false;
	}

  /// Try to read until we find IS_NUT_CONFIGURED
  try
	{
	  Glib::IOStatus retCode;
	  retCode = wizardConfFile->read_line (lineRead);
	  while ((retCode != Glib::IO_STATUS_EOF) &&
			 (retCode != Glib::IO_STATUS_ERROR))
		{
		  /// search for the field...
		  if ((test = lineRead.compare(0, 16, "bIsNutConfigured")) == 0)
			{			  
			  if (lineRead.compare(17, 4, "true") == 0)
				{
				  std::cout << "NUT is already configured" << std::endl;
				  retValue = true;
				}
			  else
				{
				  std::cout << "NUT is not configured" << std::endl;
				  retValue = false;
				}

			  break;
			}
		  else
			{
			  /// Read a new line
			  retCode = wizardConfFile->read_line (lineRead);
			}
		}
	}
  catch (Glib::IOChannelError& e)
	{
	  return false;
	}

  /// Close the file
	wizardConfFile->close();

	return retValue;
}

// Dump all configuration files
bool NutConfig::dumpConfiguration()
{
  bool retCode = false;

  cout << "Dumping configuration to NUT files..." << endl;

  // FIXME: 
  // - check and correct file permissions
  // - need to backup previous files ?!
  // - configuration might be read first ?!

  // write ups.conf
  retCode = dumpUpsConfiguration();
	
  // write upsd.conf
  retCode |= dumpUpsdConfiguration();
	
  // write upsd.users
  retCode |= dumpUpsdUsersConfiguration();
		
  // write upsmon.conf
  retCode |= dumpUpsmonConfiguration();
		
  // write upssched.conf
  retCode |= dumpUpsschedConfiguration();
		
  // write NUT default parameters
  // FIXME: Debian only
  retCode |= dumpDefaultConfiguration();

  // write PSP default parameters
  retCode |= dumpPspConfiguration();

  // write session Autostart file
  if (getDisplaySystray() == true)
	retCode |= dumpAutostartConfiguration();

  // Finally restore file permissions
  restoreFilesPermissions();

  return retCode;
}

bool NutConfig::dumpUpsConfiguration()
{
  Glib::RefPtr<Glib::IOChannel> confFile;
  std::ostringstream upsCfg;
  bool retCode;

  try {
	// Open the file, and write the PSP header
	confFile = Glib::IOChannel::create_from_file (Glib::ustring(*nutCfgDir + Glib::ustring("ups.conf")).raw(), "w+");
	confFile->write (MNC_CFG_HEADER);

	// Create the actual content
	upsCfg << "[mgeups]\n\tdriver = ";
	switch(driverType)
	{
	  case UPS_TYPE_USB:
			upsCfg << "usbhid-ups";
			break;
	  case UPS_TYPE_SHUT:
			upsCfg << "mge-shut";
			break;
	  case UPS_TYPE_UTALK:
			upsCfg << "mge-utalk";
		break;
	  	default:
	  case UPS_TYPE_NONE:
			break;
	}
	upsCfg << "\n\tport = " << communicationPort->data();
	upsCfg << "\n\tdesc = " << "\"Computer Room\"\n";

	// Workaround for missing shutoff implementation on SUSE
	if (systemType == MgeUtils::SYSTEM_TYPE_SUSE)
	  upsCfg << "\n\toffdelay = 120\n";

	// Flush and close the file
	confFile->write (upsCfg.str());
	confFile->close();

	retCode = true;

  } catch (Glib::Exception& e) 
	{ 
	  // FIXME
	} 

  return retCode;
}

bool NutConfig::dumpUpsdConfiguration()
{
  Glib::RefPtr<Glib::IOChannel> confFile;
  std::ostringstream upsdCfg;
  bool retCode;

  try {
	// FIXME (Comment): this file is generated statically,
	// there's no need (at least in PSP mode) to broaden its scope
	// like opening the supervision to the network

	// Open the file, and write the PSP header
	confFile = Glib::IOChannel::create_from_file (Glib::ustring(*nutCfgDir + Glib::ustring("upsd.conf")).raw(), "w+");
	confFile->write (MNC_CFG_HEADER);


	// FIXME (FFE): manage MAXAGE

	// Create the actual content	
	upsdCfg << "# Access Control Lists (ACLs)\n";
	upsdCfg << "ACL all 0.0.0.0/0\n";
	upsdCfg << "ACL localhost 127.0.0.1/32\n";
	upsdCfg << "ACCEPT localhost\n";
	upsdCfg << "REJECT all\n";

  	// Flush and close the file
	confFile->write (upsdCfg.str());
	confFile->close();

	retCode = true;

  } catch (Glib::Exception& e) 
	{ 
	  // FIXME
	} 

  return retCode;
}

bool NutConfig::dumpUpsdUsersConfiguration()
{
  Glib::RefPtr<Glib::IOChannel> confFile;
  std::ostringstream upsdUsersCfg;
  bool retCode;

  try {
	// FIXME (Comment): this file is generated statically,
	// there's no need (at least in PSP mode) to broaden its scope
	// like opening the supervision to the network

	// Open the file, and write the PSP header
	confFile = Glib::IOChannel::create_from_file (Glib::ustring(*nutCfgDir + Glib::ustring("upsd.users")).raw(), "w+");
	confFile->write (MNC_CFG_HEADER);

	// Create the actual content
	upsdUsersCfg << "\n[admin]\n\tpassword = mgepass\n\tallowfrom = localhost\n\tactions = SET\n\tinstcmds = ALL\n";
	upsdUsersCfg << "\n[monuser]\n\tpassword  = mgepass\n\tallowfrom = localhost\n\tupsmon master\n";

  	// Flush and close the file
	confFile->write (upsdUsersCfg.str());
	confFile->close();

	retCode = true;

  } catch (Glib::Exception& e) 
	{ 
	  // FIXME
	} 
  return retCode;
}
bool NutConfig::dumpUpsmonConfiguration()
{
  Glib::RefPtr<Glib::IOChannel> confFile;
  std::ostringstream upsmonCfg;
  bool retCode;

  try {
	// Open the file, and write the PSP header	
	confFile = Glib::IOChannel::create_from_file (Glib::ustring(*nutCfgDir + Glib::ustring("upsmon.conf")).raw(), "w+");
	confFile->write (MNC_CFG_HEADER);

	// FIXME (Comment): this file is generated statically for the moment

	// Create the actual content
	// FIXME (FFE): RUN_AS_USER nutmon
	upsmonCfg << "MONITOR mgeups@localhost 1 monuser mgepass master\n";
	upsmonCfg << "MINSUPPLIES 1\n";

	// FIXME: should be adapted to the system/user config, or defered to upssched
	switch(shutdownType)
	  {
	  case SD_NONE:
		upsmonCfg << "SHUTDOWNCMD \"\"\n";
		break;
	  case SD_CUSTOM:
	  upsmonCfg << "SHUTDOWNCMD \"" << shutdownCustomCmd->raw() << "\"\n";
		break;
	  case SD_STANDARD:
	  default:
		upsmonCfg << "SHUTDOWNCMD \"/sbin/shutdown -h +0\"\n";
		break;
	  }

	// FIXME: @sbin@ path (/sbin on Deb, /usr/sbin on Mdk...)
	switch(systemType)
	  {
	  case MgeUtils::SYSTEM_TYPE_DEBIAN:
// FIXME: should be installed in /sbin, not /usr/sbin
 		upsmonCfg << "NOTIFYCMD /sbin/upssched\n";
 		break;
	  case MgeUtils::SYSTEM_TYPE_MDK:
		case MgeUtils::SYSTEM_TYPE_MDK_2007:
	  case MgeUtils::SYSTEM_TYPE_RHFC:
	  case MgeUtils::SYSTEM_TYPE_SUSE:
	  default:
		upsmonCfg << "NOTIFYCMD /usr/sbin/upssched\n";
		break;
	  }
	upsmonCfg << "POLLFREQ 5\n";
	upsmonCfg << "POLLFREQALERT 5\n";
	upsmonCfg << "HOSTSYNC 15\n";
	upsmonCfg << "DEADTIME 15\n";
	upsmonCfg << "POWERDOWNFLAG /etc/killpower\n";

	// FIXME: NOTIFYMSG (needed?)
	// for upssched action (LOWBATT, FSD, COMMBAD, SHUTDOWN are handled by upsmon)
	// FIXME: WALL activation should be user configurable
// 	upsmonCfg << "NOTIFYFLAG ONLINE SYSLOG+WALL+EXEC\n";
// 	upsmonCfg << "NOTIFYFLAG ONBATT SYSLOG+WALL+EXEC\n";
	upsmonCfg << "NOTIFYFLAG ONLINE SYSLOG+EXEC\n";
	upsmonCfg << "NOTIFYFLAG ONBATT SYSLOG+EXEC\n";
	// FIXME: while waiting for drivers improvement
	upsmonCfg << "NOTIFYFLAG COMMOK IGNORE\n";
	upsmonCfg << "NOTIFYFLAG COMMBAD IGNORE\n";

	upsmonCfg << "\n";
	upsmonCfg << "RBWARNTIME 43200\n";
	upsmonCfg << "NOCOMMWARNTIME 300\n";
	upsmonCfg << "FINALDELAY 5\n";

  	// Flush and close the file
	confFile->write (upsmonCfg.str());
	confFile->close();

	retCode = true;

  } catch (Glib::Exception& e) 
	{ 
	  // FIXME
	} 
  return retCode;
}
bool NutConfig::dumpUpsschedConfiguration()
{
  Glib::RefPtr<Glib::IOChannel> confFile;
  std::ostringstream upsschedCfg;
  bool retCode;

  cout << "Dumping Upssched configuration to file..." << endl;

  try {
	// Open the file, and write the PSP header
	confFile = Glib::IOChannel::create_from_file (Glib::ustring(*nutCfgDir + Glib::ustring("upssched.conf")).raw(), "w+");
	confFile->write (MNC_CFG_HEADER);

	// Source the /etc/nut/wizard.conf file to obtain
	//	specific configuration (ie shutdown criteria/config)
	//upsschedCfg << "

	// Create the actual content
	// FIXME: @sbin@ path (/sbin on Deb, /usr/sbin on Mdk...)
	switch(systemType)
	  {
	  case MgeUtils::SYSTEM_TYPE_DEBIAN:
		upsschedCfg << "CMDSCRIPT /usr/sbin/upssched-cmd\n";
		// FIXME: check for exact PATH and security (create /var/run/upssched owned by upsmon user)
		upsschedCfg << "PIPEFN /var/run/nut/upssched.pipe\n";
		upsschedCfg << "LOCKFN /var/run/nut/upssched.lock\n";
		break;
	  case MgeUtils::SYSTEM_TYPE_RHFC:
		upsschedCfg << "CMDSCRIPT /usr/sbin/upssched-cmd-mge\n";
		// FIXME: check for exact PATH and security (create /var/run/upssched owned by upsmon user)
		upsschedCfg << "PIPEFN /var/lib/ups/upssched.pipe\n";
		upsschedCfg << "LOCKFN /var/lib/ups/upssched.lock\n";
		break;
	  case MgeUtils::SYSTEM_TYPE_SUSE:
		upsschedCfg << "CMDSCRIPT /usr/sbin/upssched-cmd\n";
		// FIXME: check for exact PATH and security (create /var/run/upssched owned by upsmon user)
		upsschedCfg << "PIPEFN /var/lib/ups/upssched.pipe\n";
		upsschedCfg << "LOCKFN /var/lib/ups/upssched.lock\n";
		break;
	  case MgeUtils::SYSTEM_TYPE_MDK:
		case MgeUtils::SYSTEM_TYPE_MDK_2007:
	  default:
		upsschedCfg << "CMDSCRIPT /usr/sbin/upssched-cmd\n";
		// FIXME: check for exact PATH and security (create /var/run/upssched owned by upsmon user)
		upsschedCfg << "PIPEFN /var/state/ups/upssched.pipe\n";
		upsschedCfg << "LOCKFN /var/state/ups/upssched.lock\n";
		break;
	  }
	// Event definition
	// FIXME: managed here? or through wizard.conf
	if (shutdownTimer != -1)
	  {
		upsschedCfg << "AT ONBATT * START-TIMER ups-on-battery-timer " << shutdownTimer << "\n";
		upsschedCfg << "AT ONLINE * CANCEL-TIMER ups-on-battery-timer\n";
	  }
	upsschedCfg << "AT ONBATT * EXECUTE ups-on-battery\n";
	upsschedCfg << "AT ONLINE * EXECUTE ups-back-on-line\n";
	
  	// Flush and close the file
	confFile->write (upsschedCfg.str());
	confFile->close();

	retCode = true;

  } catch (Glib::Exception& e) 
	{ 
	  // FIXME
	} 
  return retCode;
}

// store upssched params (shutdown timer, criteria, ...)
bool NutConfig::dumpPspConfiguration()
{
  std::ostringstream wizardCfg;
  Glib::RefPtr<Glib::IOChannel> confFile;
  bool retCode = false;

  cout << "Dumping PSP configuration to wizard.conf file..." << endl;

  try {
	// Open the file, and write the PSP header
	confFile = Glib::IOChannel::create_from_file (Glib::ustring(*nutCfgDir + Glib::ustring("wizard.conf")).raw(), "w+");
	confFile->write (MNC_CFG_HEADER);

	// Create the actual content
	// FIXME: bIsNutConfigured seems not valued at dump time!!
	wizardCfg << "bIsNutConfigured=" << ((bIsNutConfigured == true)?"true":"false") << "\n";
	// FIXME: SD_TIMER is handled directly by dumpUpsschedConfiguration()

	// FIXME: SD_BATT_LEVEL is only needed if > battery.charge.low
	// or deactivate the standard mechanism for upssched only !?
	// => or rien a foutre (both)
	wizardCfg << "shutdownBatteryLevel=" << shutdownBatteryLevel << "\n";
	wizardCfg << "shutdownBatteryRuntime=" << shutdownBatteryRuntime << "\n";

	wizardCfg << "displaySystray=" << ((displaySystray == true)?"true":"false") << "\n";

	// Flush and close the file
	confFile->write (wizardCfg.str());
	confFile->close();

	retCode = true;

  } catch (Glib::Exception& e) 
	{ 
	  // FIXME
	} 
  return retCode;
}

/// This function is only used on:
/// - Debian systems to generate the file /etc/default/nut
/// - RHFC systems to generate the file /etc/sysconfig/ups
bool NutConfig::dumpDefaultConfiguration()
{
  std::ostringstream defaultCfg;
  Glib::RefPtr<Glib::IOChannel> confFile;
  bool retCode = false;

  switch(systemType)
	{
	case MgeUtils::SYSTEM_TYPE_DEBIAN:
	  cout << "Dumping NUT default configuration to /etc/default/nut..." << endl;

	  try {
		// Open the file, and write the PSP header
		confFile = Glib::IOChannel::create_from_file ("/etc/default/nut", "w+");
		confFile->write (MNC_CFG_HEADER);

		// Create the actual content
		// FIXME: test if we're on a Debian
		defaultCfg << "# start upsd\nSTART_UPSD=yes\n";
		defaultCfg << "# start upsmon\nSTART_UPSMON=yes\n";
	
		// Flush and close the file
		confFile->write (defaultCfg.str());
		confFile->close();
		
		retCode = true;
		
	  } catch (Glib::Exception& e) 
		{ 
		  // FIXME
		} 
	  break;
	case MgeUtils::SYSTEM_TYPE_RHFC:
	  cout << "Dumping NUT default configuration to /etc/sysconfig/ups..." << endl;

	  try {
		// Open the file, and write the PSP header
		confFile = Glib::IOChannel::create_from_file ("/etc/sysconfig/ups", "w+");
		confFile->write (MNC_CFG_HEADER);

		// Create the actual content
		defaultCfg << "# start upsd\nSERVER=yes\n";
		defaultCfg << "# start driver\nMODEL=upsdrvctl\n";
		defaultCfg << "DEVICE=\n";
		defaultCfg << "OPTIONS=\n";
		defaultCfg << "UPSD_OPTIONS=\n";

		// Flush and close the file
		confFile->write (defaultCfg.str());
		confFile->close();
		
		retCode = true;
		
	  } catch (Glib::Exception& e) 
		{ 
		  // FIXME
		} 
	  break;	  
	default:
	  retCode = true;
	  break;
	}
  return retCode;
}


/// This function creates the autostart entries
// for the supported WindowManager
// Note: for the moment, only KDE uses it, but
// it's under standardisation (www.freedesktop.org)
bool NutConfig::dumpAutostartConfiguration()
{
  std::ostringstream autostartCfg;
  Glib::RefPtr<Glib::IOChannel> sessionFile;
  bool retCode = false;

  // RHFC already have menu entry through the package!
  // Update: this _is not_ a menu entry!!
//   if (systemType == MgeUtils::SYSTEM_TYPE_RHFC)
// 	return true;

  cout << "Dumping PSP autostart..." << endl;

  // Create the content
  autostartCfg << "[Desktop Entry]\n";
  autostartCfg << "Encoding=UTF-8\n";
  autostartCfg << "Name=Personal Solution Pac\n";
  autostartCfg << "Exec=psp --start-hidden\n";
  autostartCfg << "Icon=mgeops-psp\n";
  autostartCfg << "Type=Application\n";
  autostartCfg << "Terminal=false\n";

	try {
		// First try with the XDG method
		sessionFile = Glib::IOChannel::create_from_file ("/etc/xdg/autostart/psp.tray.desktop", "w+");
		// Flush and close the file
		sessionFile->write (autostartCfg.str());
		sessionFile->close();
	} catch (Glib::Exception& e) 
		{
			try {
				// Open the KDE file, and write the PSP header
				sessionFile = Glib::IOChannel::create_from_file ("/usr/share/gnome/autostart/psp.tray.desktop", "w+");
				// Flush and close the file
				sessionFile->write (autostartCfg.str());
				sessionFile->close();
			} catch (Glib::Exception& e) 
				{
			  	// FIXME
			  	std::cout << "Can't store Gnome Autostart configuration in /usr/share/gnome/autostart/" << std::endl;
				}
			try {
				// Open the KDE file, and write the PSP header
				sessionFile = Glib::IOChannel::create_from_file ("/usr/share/autostart/psp.tray.desktop", "w+");
				// Flush and close the file
				sessionFile->write (autostartCfg.str());
				sessionFile->close();
			} catch (Glib::Exception& e) 
				{
					try {
					// Open the KDE file, and write the PSP header
						sessionFile = Glib::IOChannel::create_from_file ("/opt/kde3/share/autostart/psp.tray.desktop", "w+");
					// Flush and close the file
						sessionFile->write (autostartCfg.str());
						sessionFile->close();
					} catch (Glib::Exception& e) 
						{ 
							// FIXME
							std::cout << "Can't store KDE Autostart configuration" << std::endl;
						} 
				}
	
		  try {
				// Open the file, and write the PSP header
				sessionFile = Glib::IOChannel::create_from_file ("~/.gnome2/panel2.d/default/launchers/psp.tray.desktop", "w+");
				// Flush and close the file
				sessionFile->write (autostartCfg.str());
				sessionFile->close();
		  } catch (Glib::Exception& e) 
			{ 
			  // FIXME
			  std::cout << "Can't store Gnome Autostart configuration" << std::endl;
			} 
	}

  retCode = true;
  return retCode;
}


void NutConfig::restoreFilesPermissions()
{
  std::string outChannel("/dev/null");
  Glib::ustring *commandLine;
  int exitCode = 0;

  // FIXME: need owner + group
  // Debian => chown root:nut /etc/nut/*
  // Mdk => chown root:ups /etc/ups/*
  // RH/FC: chown root:nut /etc/ups/*
  restoreFilePermissions("ups.conf", "640");	// same as rw-r-----
  restoreFilePermissions("upsd.conf", "640");	// same as rw-r-----
  restoreFilePermissions("upsd.users", "640");	// same as rw-r-----
  restoreFilePermissions("upsmon.conf", "640");	// same as rw-r-----
  // These 2 files need to be user readable for the Config View
  restoreFilePermissions("upssched.conf", "644"); // same as rw-r--r--
  restoreFilePermissions("wizard.conf", "644");	  // same as rw-r--r--


  // chown the whole NUT dir
  try
	{
	  Glib::spawn_command_line_sync (Glib::ustring("/bin/chown -R root:" + *nutUser + " " + *nutCfgDir).raw());
// 	  Glib::spawn_command_line_sync (Glib::ustring("/bin/chown -R root:" + *nutUser + " " + *nutCfgDir).raw(),
// 									 &outChannel, // std output
// 									 &outChannel, // std error
// 									 &exitCode);
	} catch (Glib::SpawnError& e)
	{
	}
  std::cout << "Exit code for chown is: " << exitCode << std::endl;
}

// Restore one file's permission
void NutConfig::restoreFilePermissions(std::string fileName, std::string filePermissions)
{
  std::string outChannel("/dev/null");
  int exitCode = 0;

  // FIXME: address the non starting driver!!!
  try
	{
	  Glib::spawn_command_line_sync (Glib::ustring("/bin/chmod " + Glib::ustring(filePermissions) + " " + *nutCfgDir + Glib::ustring(fileName)).raw(), //&commandLine,
									 &outChannel, // std output
									 &outChannel, // std error
									 &exitCode);
	} catch (Glib::SpawnError& e)
	{
	}
  std::cout << "Exit code is: " << exitCode << std::endl;
}

void NutConfig::setDriverType(int newValue)
{
  driverType = newValue;
}

void NutConfig::setDriverType(Glib::ustring *newValue)
{
  if (newValue->compare("usbhid-ups") == 0)
	setDriverType(UPS_TYPE_USB);
  else
	{
		if (newValue->compare("mge-shut") == 0)
			setDriverType(UPS_TYPE_SHUT);
	  else
		{
		  if (newValue->compare("mge-utalk") == 0)
				setDriverType(UPS_TYPE_UTALK);
		  else
				setDriverType(UPS_TYPE_NONE);
		}
	}
}

void NutConfig::setCommunicationPort(Glib::ustring *newValue)
{
  communicationPort->assign(*newValue);
}

void NutConfig::setShutdownTimer(int newValue)
{
  shutdownTimer = newValue;
  cout << "ShutdownTimer = " << shutdownTimer  << endl;
}

int NutConfig::getShutdownTimer()
{
  return shutdownTimer;
}

void NutConfig::setShutdownBatteryLevel(int newValue)
{
  shutdownBatteryLevel = newValue;
  cout << "setShutdownBatteryLevel = " << shutdownBatteryLevel << endl;
}

int NutConfig::getShutdownBatteryLevel()
{
  cout << "getShutdownBatteryLevel = " << shutdownBatteryLevel << endl;
  return shutdownBatteryLevel;
}

void NutConfig::setShutdownBatteryRuntime(int newValue)
{
  shutdownBatteryRuntime = newValue;
  cout << "ShutdownRuntime = " << shutdownBatteryRuntime << endl;
}

int NutConfig::getShutdownBatteryRuntime()
{
  return shutdownBatteryRuntime;
}


void NutConfig::setShutdownType(int newValue)
{
  shutdownType = newValue;
}

int NutConfig::getShutdownType()
{
  return shutdownType;
}

void NutConfig::setCustomShutdownCommand(Glib::ustring newCommand)
{
  delete shutdownCustomCmd;
  shutdownCustomCmd = new Glib::ustring(newCommand);
}

Glib::ustring *NutConfig::getCustomShutdownCommand()
{
  return new Glib::ustring(*shutdownCustomCmd);
}


bool NutConfig::readUpsConfiguration()
{
  // need to be read?
  return true;
}

bool NutConfig::readUpsdConfiguration()
{
  // need to be read?
  return true;
}

bool NutConfig::readUpsdUsersConfiguration()
{
  // need to be read?
  return true;
}

bool NutConfig::readUpsmonConfiguration()
{
  Glib::RefPtr<Glib::IOChannel> confFile;
  Glib::ustring lineRead;
  bool retValue = false;
  int test;

  /// Open the wizard.conf file
  try
	{
	  confFile = Glib::IOChannel::create_from_file(Glib::ustring(*nutCfgDir + Glib::ustring("upsmon.conf")).raw(), "r");
	}
  catch (Glib::FileError& e)
	{
	  std::cout << "Warning: can't read upsmon.conf" << std::endl;
	  return false;
	}

  /// Try to read SHUTDOWNCMD
  try
	{
	  Glib::IOStatus retCode;
	  Glib::ustring *lineTag;

	  retCode = confFile->read_line (lineRead);

	  while ((retCode != Glib::IO_STATUS_EOF) &&
			 (retCode != Glib::IO_STATUS_ERROR))
		{
		  // search for the field...
		  lineTag = new Glib::ustring("SHUTDOWNCMD");

		  if ((test = lineRead.compare(0, lineTag->length(), *lineTag)) == 0)
			{
			  // Check the shudown type
			  if (lineRead.compare("SHUTDOWNCMD \"/sbin/shutdown -h +0\"\n") == 0)
				setShutdownType(SD_STANDARD);
			  else
				{
				  if (lineRead.compare("SHUTDOWNCMD \"\"\n") == 0)
					setShutdownType(SD_NONE);
				  else
					{
					  setShutdownType(SD_CUSTOM);

					  // Extract the custom shutdown command
					  Glib::ustring::size_type idxSdCmd = lineRead.Glib::ustring::find ('"');
					  if (idxSdCmd != -1)
						setCustomShutdownCommand(lineRead.substr(idxSdCmd+1, (lineRead.rfind ('"') - idxSdCmd) - 1 ));
					}
				}
			  std::cout << "SHUTDOWNCMD = " << shutdownCustomCmd->raw() << std::endl;
			  retValue = true;
			}

		  // Read a new line
		  retCode = confFile->read_line (lineRead);

		} // whend
	  
	  retValue = false;
	}
  catch (Glib::IOChannelError& e)
	{
	  retValue = false;
	}

	/// Close the file
	confFile->close();

	return retValue;
}

bool NutConfig::readUpsschedConfiguration()
{
  Glib::RefPtr<Glib::IOChannel> confFile;
  Glib::ustring lineRead;
  bool retValue = false;
  int test;

  /// Open the wizard.conf file
  try
	{
	  confFile = Glib::IOChannel::create_from_file(Glib::ustring(*nutCfgDir + Glib::ustring("upssched.conf")).raw(), "r");
	}
  catch (Glib::FileError& e)
	{
	  return false;
	}

  /// Try to read shutdownTimer
  try
	{
	  Glib::IOStatus retCode;
	  Glib::ustring *lineTag;

	  retCode = confFile->read_line (lineRead);

	  while ((retCode != Glib::IO_STATUS_EOF) &&
			 (retCode != Glib::IO_STATUS_ERROR))
		{
		  // search for the field...
		  lineTag = new Glib::ustring("AT ONBATT * START-TIMER ups-on-battery-timer");

		  if ((test = lineRead.compare(0, lineTag->length(), *lineTag)) == 0)
			{
			  setShutdownTimer(atoi(lineRead.substr(lineTag->length() + 1).c_str()));
			  std::cout << "shutdownTimer = " << getShutdownTimer() << std::endl;
			}

		  // Read a new line
		  retCode = confFile->read_line (lineRead);

		} // whend
	  
	  retValue = true;
	}
  catch (Glib::IOChannelError& e)
	{
	  retValue = false;
	}

  /// Close the file
	confFile->close();

	return retValue;
}

bool NutConfig::readPspConfiguration()
{
  Glib::RefPtr<Glib::IOChannel> wizardConfFile;
  Glib::ustring lineRead;
  bool retValue = false;
  int test;

  /// Open the wizard.conf file
  try
	{
	  wizardConfFile = Glib::IOChannel::create_from_file(Glib::ustring(*nutCfgDir + Glib::ustring("wizard.conf")).raw(), "r");
	}
  catch (Glib::FileError& e)
	{
	  return false;
	}

  /// Try to read all settings
  try
	{
	  Glib::IOStatus retCode;
	  retCode = wizardConfFile->read_line (lineRead);

	  while ((retCode != Glib::IO_STATUS_EOF) &&
			 (retCode != Glib::IO_STATUS_ERROR))
		{
		  // search for the fields...
		  if ((test = lineRead.compare(0, 16, "bIsNutConfigured")) == 0)
			{			  
			  if (lineRead.compare(17, 4, "true") == 0)
				setIsNutConfigured(true);
			  else
				setIsNutConfigured(false);

			  std::cout << "NUT is " << ((getIsNutConfigured()==true)?"already":"not") << " configured" << std::endl;
			}

		  if ((test = lineRead.compare(0, 20, "shutdownBatteryLevel")) == 0)
			{		  
			  if (lineRead.compare(21, 2, "-1") != 0)
					setShutdownBatteryLevel(atoi(lineRead.substr(21).c_str()));

			  std::cout << "shutdownBatteryLevel = " << getShutdownBatteryLevel() << std::endl;
			}

		  if ((test = lineRead.compare(0, 22, "shutdownBatteryRuntime")) == 0)
			{		  
			  if (lineRead.compare(23, 2, "-1") != 0)
				setShutdownBatteryRuntime(atoi(lineRead.substr(23).c_str()));

			  std::cout << "shutdownBatteryRuntime = " << getShutdownBatteryRuntime() << std::endl;
			}

		  if ((test = lineRead.compare(0, 14, "displaySystray")) == 0)
			{	 
			  if (lineRead.compare(15, 4, "true") == 0)
				setDisplaySystray(true);
			  else
				setDisplaySystray(false);
	  
			  std::cout << "System tray is " << ((getDisplaySystray()==false)?"not ":"") << "visible" << std::endl;
			}
		  
		  // Read a new line
		  retCode = wizardConfFile->read_line (lineRead);

		} // whend 
	}
  catch (Glib::IOChannelError& e)
	{
	  return false;
	}

  /// Close the file
	wizardConfFile->close();

	return retValue;
}

bool NutConfig::readDefaultConfiguration()
{
  // need to be read?
  return true;
}


bool NutConfig::readConfiguration()
{
  bool retCode = false;

  cout << "Reading configuration from NUT files..." << endl;

  // read ups.conf
  retCode = readUpsConfiguration();
	
  // read upsd.conf
  retCode |= readUpsdConfiguration();
	
  // read upsd.users
  retCode |= readUpsdUsersConfiguration();
		
  // read upsmon.conf
  retCode |= readUpsmonConfiguration();
		
  // read upssched.conf
  retCode |= readUpsschedConfiguration();

  // read PSP default parameters
  retCode |= readPspConfiguration();

  return retCode;
}
