// A Network UPS Tools Configuration interface. 
/**
 * This class allows to manipulate NUT configuration files.
 *
 * @author Arnaud Quette <arnaud.quette@mgeups.com>
 * @version 2004/12/21 Creation
 *
 * Copyright (C) 2004 - 2005 by MGE UPS SYSTEMS
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2.
 *  Please, read the COPYING file provided with this software for the full text
 */


#ifndef _NUTCONFIG_HH
#  include <glibmm.h>
#  define _NUTCONFIG_HH

#include <config.h> 
#include "tool.h"
#include "MgeUtils.h"
#include "NutClient.h" // for UPS_TYPE_*


class NutConfig : public Tool
{
 public:
  // Retrieve NUT configuration status from wizard.conf file
  bool isNutConfigured();

  // Constructors
  NutConfig ();
  NutConfig (int drvType, Glib::ustring* comPort);

  ~NutConfig () { };
  
  enum configFileType { 
   CONFIG_NONE = 0, 
   CONFIG_UPS,  
   CONFIG_UPSD,  
   CONFIG_UPSD_USERS,  
   CONFIG_UPSMON,   
   CONFIG_UPSSCHED,   
   CONFIG_ALL 
  };
  
  enum sdType {
	SD_NONE,
	SD_STANDARD,	// halt
	SD_CUSTOM		// implies shutdownCustomCmd
  };

 private:
  int driverType;
  Glib::ustring* communicationPort;
  int shutdownTimer;
  int shutdownBatteryLevel;
  int shutdownBatteryRuntime;
  bool bIsNutConfigured;
  int shutdownType;
  Glib::ustring* shutdownCustomCmd;
  int systemType;
  MgeUtils pspUtils;
  Glib::ustring* nutCfgDir;
  Glib::ustring* nutUser;
  bool displaySystray;

 public:
  // Values accessors
  int getDriverType() { return driverType; }
  void setDriverType(int newValue);
  void setDriverType(Glib::ustring *newValue);
  void setDisplaySystray(bool visibility)
  {
	displaySystray = visibility; 
  }
  bool getDisplaySystray() { return (displaySystray==true)?true:false; }
  void setCommunicationPort(Glib::ustring *newValue);
  bool getIsNutConfigured() { return bIsNutConfigured; }
  void setIsNutConfigured(bool newValue)
  {
	bIsNutConfigured = newValue;
  }

  void setShutdownTimer(int newValue);
  int getShutdownTimer();
  void setShutdownBatteryLevel(int newValue);
  int getShutdownBatteryLevel();
  void setShutdownBatteryRuntime(int newValue);
  int getShutdownBatteryRuntime();
  void setShutdownType(int newValue);
  int getShutdownType();
  void setCustomShutdownCommand(Glib::ustring newCommand);
  Glib::ustring *getCustomShutdownCommand();

  // Configuration files serialisation (Write Only!)
  bool dumpConfiguration();
  bool dumpUpsConfiguration();
  bool dumpUpsdConfiguration();
  bool dumpUpsdUsersConfiguration();
  bool dumpUpsmonConfiguration();
  bool dumpUpsschedConfiguration();
  bool dumpPspConfiguration();
  bool dumpDefaultConfiguration();
  bool dumpAutostartConfiguration();

  void restoreFilesPermissions();
  void restoreFilePermissions(std::string fileName, std::string filePermissions);


  // Configuration files serialisation (Read Only!)
  bool readConfiguration();
  bool readUpsConfiguration();
  bool readUpsdConfiguration();
  bool readUpsdUsersConfiguration();
  bool readUpsmonConfiguration();
  bool readUpsschedConfiguration();
  bool readPspConfiguration();
  bool readDefaultConfiguration();
  
};

#endif
