
#include <iostream>
#include <config.h>
#include "MgeUtils.h"
#include "SystemService.h"

#define INITD_PATH "/etc/init.d/"

SystemService::SystemService()
{
  // Base init.d path (ok for Deb, Mdk, SUSE, ?RH?): /etc/init.d/

  // Debian: 
  // 	- service: ups-monitor
  // Mdk:
  // 	- service: upsd / upsmon
  // RH / SUSE: idem Mdk ??
  systemType = pspUtils.getSystemType();
}

/// Only start the driver (the service wraps it)
bool SystemService::startDriver()
{
  std::string outChannel("/dev/null");
  bool retCode = false;
  int exitCode = 0;

  // FIXME: address the non starting driver!!!
  try
	{
	  // Debian, Mdk: /sbin/upsdrvctl
	  // SUSE, RHFC ?
	  Glib::spawn_command_line_sync ("/sbin/upsdrvctl start mgeups", //&commandLine,
									 &outChannel, // std output
									 &outChannel, // std error
									 &exitCode);
	} catch (Glib::SpawnError& e)
	{
	}
  std::cout << "Exit code is: " << exitCode << std::endl;
  // test exitCode => retCode
  return true;
}

/// Only stop the driver (the service wraps it)
bool SystemService::stopDriver()
{
  std::string outChannel("/dev/null");
  bool retCode = false;
  int exitCode = 0;

  // FIXME: address the non starting driver!!!
  try
	{
	  Glib::spawn_command_line_sync ("/sbin/upsdrvctl stop mgeups", //&commandLine,
									 &outChannel, // std output
									 &outChannel, // std error
									 &exitCode);
	} catch (Glib::SpawnError& e)
	{
	}
  std::cout << "Exit code is: " << exitCode << std::endl;
  // test exitCode => retCode
  return true;
}

/// Only restart the driver (the service wraps it)
bool SystemService::restartDriver()
{
  bool retCode = false;

  retCode = stopDriver();
  retCode |= startDriver();

  return retCode;
}

bool SystemService::startStopService(char *command)
{
  std::string outChannel("/dev/null");
  bool retCode = false;
  int exitCode = 0;

  // FIXME: address the non starting driver!!!
  try
	{
	  switch(systemType)
		{
		case MgeUtils::SYSTEM_TYPE_DEBIAN:
		  Glib::spawn_command_line_sync (Glib::ustring(INITD_PATH "nut " + Glib::ustring(command)).raw(),
										 &outChannel, // std output
										 &outChannel, // std error
										 &exitCode);
		  break;
		case MgeUtils::SYSTEM_TYPE_MDK:
		case MgeUtils::SYSTEM_TYPE_MDK_2007:
		  if (Glib::ustring(command).compare("start") == 0)
			{
			  Glib::spawn_command_line_sync (Glib::ustring(INITD_PATH "upsd " + Glib::ustring(command)).raw(),
											 &outChannel, // std output
											 &outChannel, // std error
											 &exitCode);
			  Glib::spawn_command_line_sync (Glib::ustring(INITD_PATH "upsmon " + Glib::ustring(command)).raw(),
											 &outChannel, // std output
											 &outChannel, // std error
											 &exitCode);
			}
		  else
			{
			  Glib::spawn_command_line_sync (Glib::ustring(INITD_PATH "upsmon " + Glib::ustring(command)).raw(),
											 &outChannel, // std output
											 &outChannel, // std error
											 &exitCode);
			  Glib::spawn_command_line_sync (Glib::ustring(INITD_PATH "upsd " + Glib::ustring(command)).raw(),
											 &outChannel, // std output
											 &outChannel, // std error
											 &exitCode);
			}
		  break;
		case MgeUtils::SYSTEM_TYPE_SUSE:
		  Glib::spawn_command_line_sync (Glib::ustring(INITD_PATH "upsd " + Glib::ustring(command)).raw(),
										 &outChannel, // std output
										 &outChannel, // std error
										 &exitCode);
		  break;
		case MgeUtils::SYSTEM_TYPE_RHFC:
		  Glib::spawn_command_line_sync (Glib::ustring(INITD_PATH "ups " + Glib::ustring(command)).raw(),
										 &outChannel, // std output
										 &outChannel, // std error
										 &exitCode);
		  break;
		default:
		  break;
		}
	} catch (Glib::SpawnError& e)
	{
  		std::cout << "startStopService got an exception" << std::endl;
	}
  std::cout << "startStopService exit code is: " << exitCode << std::endl;
  // FIXME: test exitCode => retCode
  return true;
}

bool SystemService::startService()
{
  return startStopService("start");
}
bool SystemService::stopService()
{
  bool retCode = startStopService("stop");

  // Workaround to ensure all drivers have stopped
  Glib::spawn_command_line_sync ("/usr/bin/killall -9 -q -w newhidups usbhid-ups mge-utalk mge-shut newmge-shut");

  return retCode;
}

bool SystemService::restartService()
{
  bool retCode = false;
  // FIXME: use "ups-monitor restart" or "ups-monitor reload"
  retCode = stopService();
  sleep (10);
  retCode |= startService();

  return retCode;
}

/// Simply reload configuration (post change)
bool SystemService::reloadService()
{
  return startStopService("reload");
}

bool SystemService::isServiceStarted()
{
  bool retCode = false;

  // 1) try pidof => all but need code
  // 2) else try /var/run/${basename}.pid => Debian, Mdk
  // 3) else try /var/lock/subsys/${basename} => Mdk

  //file_test (const std::string& filename, FileTest test)
  //file_get_contents (const std::string& filename)
  switch(systemType)
	{
	case MgeUtils::SYSTEM_TYPE_DEBIAN:
	  if ((Glib::file_test ("/var/run/nut/upsd.pid", Glib::FILE_TEST_EXISTS) == true)
		  && (Glib::file_test ("/var/run/nut/upsmon.pid", Glib::FILE_TEST_EXISTS) == true))
		retCode = true;
	  else
		retCode = false;
	  break;
	case MgeUtils::SYSTEM_TYPE_MDK:
	case MgeUtils::SYSTEM_TYPE_MDK_2007:
	  //if ((Glib::file_test ("/var/state/ups/upsd.pid", Glib::FILE_TEST_EXISTS) == true)
		//  && (Glib::file_test ("/var/state/ups/upsmon.pid", Glib::FILE_TEST_EXISTS) == true))
	  if ((Glib::file_test ("/var/lock/subsys/upsd", Glib::FILE_TEST_EXISTS) == true)
		  && (Glib::file_test ("/var/lock/subsys/upsmon", Glib::FILE_TEST_EXISTS) == true))
		retCode = true;
	  else
		retCode = false;
	  break;
	case MgeUtils::SYSTEM_TYPE_SUSE:
	case MgeUtils::SYSTEM_TYPE_RHFC:
	default:
	  // FIXME: test if good!?
	  //if ((Glib::file_test ("/var/lib/ups/upsd.pid", Glib::FILE_TEST_EXISTS) == true)
	  //	  && (Glib::file_test ("/var/lib/ups/upsmon.pid", Glib::FILE_TEST_EXISTS) == true))
	  if (Glib::file_test ("/var/lock/subsys/ups", Glib::FILE_TEST_EXISTS) == true)
		retCode = true;
	  else
		retCode = false;
	  break;
	}
  if (retCode == true)
	std::cout << "Services are already started" << std::endl;
  else
	std::cout << "Services are not started" << std::endl;

  return retCode; 
}
